from typing import List, Optional
from uuid import UUID

from hrthy_core.security.exceptions import MissingPermissionException, MissingScopeConfiguration
from hrthy_core.security.scopes_hierarchy import HIERARCHY_SCOPES
from hrthy_core.security.security import Requester, RequesterType
from hrthy_core.security.types import ScopeType


class ScopeChecker:
    @staticmethod
    def is_service_user(requester: Requester) -> bool:
        return requester.requester_type != RequesterType.service

    @staticmethod
    def has_all_permissions(requester: Requester, scopes: List[str]) -> bool:
        return all(s in requester.scopes for s in scopes)

    @staticmethod
    def has_at_least_one_permission(requester: Requester, scopes: List[str]) -> bool:
        return any(s in requester.scopes for s in scopes)

    @staticmethod
    def get_first_match_permission(requester: Requester, scopes: List[str]) -> Optional[str]:
        for s in scopes:
            if s in requester.scopes:
                return s
        return None

    @staticmethod
    def get_target_company(requester: Requester, priority_scopes: List[str], company_id: UUID = None) -> UUID:
        scope = ScopeChecker.get_first_match_permission(requester=requester, scopes=priority_scopes)
        if scope:
            scope_configuration = HIERARCHY_SCOPES.get(scope, None)
            if not scope_configuration:
                raise MissingScopeConfiguration()
            if scope_configuration.get('type') != ScopeType.GLOBAL \
                    and company_id is not None \
                    and company_id != requester.company_id:
                raise MissingPermissionException()
            return company_id if company_id is not None else requester.company_id
        raise MissingPermissionException()
